package query;

import global.Minibase;
import global.RID;
import global.SearchKey;
import heap.HeapFile;
import index.HashIndex;
import parser.AST_CreateIndex;
import relop.FileScan;
import relop.Schema;
import relop.Tuple;

/**
 * Execution plan for creating indexes.
 */
class CreateIndex implements Plan {

	/** Name of the index to create. */
	protected String	fileName;

	/** Name of the table to index. */
	protected String	ixTable;
	
	/** Schema of the underlying table */
	private Schema schema;

	/** Name of the column to index. */
	protected String	ixColumn;

	/**
	 * Optimizes the plan, given the parsed query.
	 * 
	 * @throws QueryException
	 *             if index already exists or table/column invalid
	 */
	public CreateIndex(AST_CreateIndex tree) throws QueryException {
		//Check if index exists
		fileName = tree.getFileName();
		QueryCheck.fileNotExists(fileName);

		//Check if underlying table exists
		ixTable = tree.getIxTable();
		schema = QueryCheck.tableExists(ixTable);
		
		//Check if column in underlying table exists
		ixColumn = tree.getIxColumn();
		QueryCheck.columnExists(schema,  ixColumn);
		
	} // public CreateIndex(AST_CreateIndex tree) throws QueryException

	/**
	 * Executes the plan and prints applicable output.
	 */
	public void execute() {
		HashIndex i = new HashIndex(fileName);
		
		//Build index by inserting tuples into it
		FileScan scan = new FileScan(schema, new HeapFile(ixTable));
		RID r;
		while(scan.hasNext())
		{
			Tuple t = scan.getNext();
			r = scan.getLastRID();
			i.insertEntry(new SearchKey(t.getField(ixColumn)), r);
		}
		scan.close();
		
		Minibase.SystemCatalog.createIndex(fileName, ixTable, ixColumn);
		System.out.println("Index (" + fileName + ") on (" + ixTable + "." + ixColumn + ") created.");
	} // public void execute()

} // class CreateIndex implements Plan
