package query;

import global.AttrType;
import parser.AST_Describe;
import relop.Schema;

/**
 * Execution plan for describing tables.
 */
class Describe implements Plan {

	/** Name of the table to describe. */
	protected String fileName;
	
	private Schema schema;

	/**
	 * Optimizes the plan, given the parsed query.
	 * 
	 * @throws QueryException
	 *             if table doesn't exist
	 */
	public Describe(AST_Describe tree) throws QueryException {
		//Validate table name
		fileName = tree.getFileName();
		schema = QueryCheck.tableExists(fileName);
	} // public Describe(AST_Describe tree) throws QueryException

	/**
	 * Executes the plan and prints applicable output.
	 */
	public void execute() {
		String desc = "DESCRIBE TABLE " + fileName;
		System.out.println(desc);
		for(int i = 0; i < desc.length(); i++)
		{
			System.out.print("-");
		}
		System.out.println();
		for(int i = 0; i < schema.getCount(); i++)
		{
			String fldName = schema.fieldName(i);
			int fldType = schema.fieldType(i);
			String fldLength = fldType == AttrType.STRING ? "(" + schema.fieldLength(i) + ")" : "";
			System.out.print(fldName + ":" + AttrType.toString(fldType) + fldLength + "\t");
		}
		System.out.println();

	} // public void execute()

} // class Describe implements Plan
