package query;

import global.Minibase;
import global.RID;
import global.SearchKey;
import heap.HeapFile;
import index.HashIndex;
import parser.AST_Insert;
import relop.Schema;
import relop.Tuple;

/**
 * Execution plan for inserting tuples.
 */
class Insert implements Plan {

	protected String fileName;
	protected Object[] values;
	private Schema t_schema;

	/**
	 * Optimizes the plan, given the parsed query.
	 * 
	 * @throws QueryException
	 *             if table doesn't exists or values are invalid
	 */
	public Insert(AST_Insert tree) throws QueryException {
		fileName = tree.getFileName();
		t_schema = QueryCheck.tableExists(fileName);

		values = tree.getValues();
		QueryCheck.insertValues(t_schema, values);
	} // public Insert(AST_Insert tree) throws QueryException

	/**
	 * Executes the plan and prints applicable output.
	 */
	public void execute() {
		Tuple t = new Tuple(t_schema, values);
		HeapFile hf = new HeapFile(fileName);
		RID rid = hf.insertRecord(t.getData());

		IndexDesc[] indices = Minibase.SystemCatalog.getIndexes(fileName);
		for (IndexDesc ind : indices)
			new HashIndex(ind.indexName).insertEntry(new SearchKey(t.getField(ind.columnName)), rid);

		// Update system catalog for relation
		RID fRID = Minibase.SystemCatalog.getFileRID(fileName, true);
		Tuple rel_cat = new Tuple(Minibase.SystemCatalog.s_rel, Minibase.SystemCatalog.f_rel.selectRecord(fRID));
		rel_cat.setField(1, rel_cat.getIntFld(1) + 1);
		Minibase.SystemCatalog.f_rel.updateRecord(fRID, rel_cat.getData());

		// print the output message
		System.out.println("1 row inserted into table " + fileName);

		// print the output message
	} // public void execute()

} // class Insert implements Plan
